@@ -17,8 +17,17 @@
 #include <QImage>
 #include <QLoggingCategory>
 
-Q_DECLARE_LOGGING_CATEGORY(LOG_IFFPLUGIN)
-Q_LOGGING_CATEGORY(LOG_IFFPLUGIN, "kf.imageformats.plugins.scitex", QtWarningMsg)
+Q_LOGGING_CATEGORY(LOG_SCTPLUGIN, "kf.imageformats.plugins.scitex", QtWarningMsg)
+
+/* *** SCT_MAX_IMAGE_WIDTH and SCT_MAX_IMAGE_HEIGHT ***
+ * The maximum size in pixel allowed by the plugin.
+ */
+#ifndef SCT_MAX_IMAGE_WIDTH
+#define SCT_MAX_IMAGE_WIDTH KIF_LARGE_IMAGE_PIXEL_LIMIT
+#endif
+#ifndef SCT_MAX_IMAGE_HEIGHT
+#define SCT_MAX_IMAGE_HEIGHT SCT_MAX_IMAGE_WIDTH
+#endif
 
 #define CTRLBLOCK_SIZE 256
 #define PRMSBLOCK_SIZE_CT 256
@@ -140,7 +149,7 @@ public:
             return false;
         }
         // Set a reasonable upper limit
-        if (width() > 300000 || height() > 300000) {
+        if (width() > SCT_MAX_IMAGE_WIDTH || height() > SCT_MAX_IMAGE_HEIGHT) {
             return false;
         }
         return m_cb.fileType() == QStringLiteral(FILETYPE_CT) && format() != QImage::Format_Invalid;
@@ -207,7 +216,7 @@ public:
         auto v = QString::fromLatin1(pchar_t(res.data()), res.size()).toDouble(&ok);
         if (ok && v > 0) {
             if (m_pb._unitsOfMeasurement) { // Inches
-                return qRoundOrZero(width() / v / 25.4 * 1000);
+                return dpi2ppm(width() / v);
             }
             // Millimeters
             return qRoundOrZero(width() / v * 1000);
@@ -221,7 +230,7 @@ public:
         auto v = QString::fromLatin1(pchar_t(res.data()), res.size()).toDouble(&ok);
         if (ok && v > 0) {
             if (m_pb._unitsOfMeasurement) { // Inches
-                return qRoundOrZero(height() / v / 25.4 * 1000);
+                return dpi2ppm(height() / v);
             }
             // Millimeters
             return qRoundOrZero(height() / v * 1000);
@@ -321,7 +330,7 @@ bool ScitexHandler::canRead() const
 bool ScitexHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("ScitexHandler::canRead() called with no device");
+        qCWarning(LOG_SCTPLUGIN) << "ScitexHandler::canRead() called with no device";
         return false;
     }
     ScitexHandlerPrivate hp;
@@ -335,7 +344,7 @@ bool ScitexHandler::read(QImage *image)
 {
     auto dev = device();
     if (dev == nullptr) {
-        qWarning("ScitexHandler::read() called with no device");
+        qCWarning(LOG_SCTPLUGIN) << "ScitexHandler::read() called with no device";
         return false;
     }
     if (!d->loadHeader(dev)) {
